
import "Turbine";
import "Turbine.Gameplay";
import "Turbine.UI";
import "Turbine.UI.Extensions";
import "Turbine.UI.Lotro";
--import "Turbine.Utils";

-- --------------------------------------------------------------
-- CalendarEntry is a simple container class that holds and tracks
-- information about an "entry" on the calendar
-- --------------------------------------------------------------

CalendarEntry = class( Turbine.Object );

function CalendarEntry:Constructor()
	Turbine.Object.Constructor( self );

  self.DateEntry = "";
  self.TimeEntry = "";
  self.Description = "";
end

-- --------------------------------------------------------------
-- CalendarEntryEditor is the editor window that allows the user
-- to change the time of the event or update it's description.
-- the user can also set an alarm / reminder that the time is
-- near.
--
-- ToDo (must wait for API updates)!
-- add a "notify" checkbox to let the user know 15 minutes before the event time
-- add an "alarm" button to alert the user audibly when the event time has been reached
-- AM vs PM?  right now, it is a 24 hour clock
-- 
-- --------------------------------------------------------------

CalendarEntryEditor = class( Turbine.UI.Lotro.Window);

-- --------------------------------------------------------------
-- Constructor
--
-- Description: this is where the window is laid out, the fields
-- and buttons are created, and events are trapped.
-- --------------------------------------------------------------
function CalendarEntryEditor:Constructor()
	Turbine.UI.Lotro.Window.Constructor( self );

	-- load the last settings saved (we will use the position to locate the editor)
	self:LoadSettings();
	self.hours = 12;
	self.minutes = 0;
	
	-- --------------------------------------------
	-- window layout
	-- --------------------------------------------
	
	-- set window properties
	self:SetSize( 400, 250 );
	self:SetBackColor( Turbine.UI.Color() );
	self:SetPosition( self.settings.positionX, self.settings.positionY );
	self:SetText( "Update Entry..." );
	self:SetOpacity( 1 );
	self:SetAllowDrop( false );
	
	-- create the date field
	self.dateLabel = Turbine.UI.Label();
	self.dateLabel:SetFont( Turbine.UI.Lotro.Font.Verdana16 );
	self.dateLabel:SetParent( self );
	self.dateLabel:SetPosition( 20, 40 );
	self.dateLabel:SetMultiline( false );	
	self.dateLabel:SetSelectable( false );	
	self.dateLabel:SetSize( 180, 20 );
	self.dateLabel:SetText( "Date:" );
	self.dateLabel:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleLeft );
	self.dateLabel:SetVisible( true );
	
	-- create the time field
	self.timeLabel = Turbine.UI.Label();
	self.timeLabel:SetFont( Turbine.UI.Lotro.Font.Verdana16 );
	self.timeLabel:SetParent( self );
	self.timeLabel:SetPosition( 190, 40 );
	self.timeLabel:SetMultiline( false );	
	self.timeLabel:SetSelectable( false );	
	self.timeLabel:SetSize( 40, 20 );
	self.timeLabel:SetText( "Time: " );
	self.timeLabel:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleLeft );
	self.timeLabel:SetVisible( true );
	
	self.hoursUp = Turbine.UI.Button();
	self.hoursUp:SetBackground( "FrostyPlugins/Calendar/Resources/UpArrow.tga" );
	self.hoursUp:SetBlendMode( Turbine.UI.BlendMode.Overlay );
	self.hoursUp:SetParent( self );
	self.hoursUp:SetPosition( 230, 30 );
	self.hoursUp:SetSize( 20, 20 );
	self.hoursUp:SetVisible( true );

	self.hoursUp.MouseEnter = function( sender, args )
		sender:SetBackground( "FrostyPlugins/Calendar/Resources/UpArrowHighlight.tga" );
		self.hoursField:SetForeColor( Turbine.UI.Color( 1, 1, 0 ) );
	end
	self.hoursUp.MouseLeave = function( sender, args )
		sender:SetBackground( "FrostyPlugins/Calendar/Resources/UpArrow.tga" );
		self.hoursField:SetForeColor( Turbine.UI.Color( 1, 1, 1 ) );
	end

	self.hoursUp.Click = function( sender, args )
		self:AdjustHours( 1 );
	end
	
	self.hoursDown = Turbine.UI.Button();
	self.hoursDown:SetBackground( "FrostyPlugins/Calendar/Resources/DownArrow.tga" );
	self.hoursDown:SetBlendMode( Turbine.UI.BlendMode.Overlay );
	self.hoursDown:SetParent( self );
	self.hoursDown:SetPosition( 230, 50 );
	self.hoursDown:SetSize( 20, 20 );
	self.hoursDown:SetVisible( true );

	self.hoursDown.MouseEnter = function( sender, args )
		sender:SetBackground( "FrostyPlugins/Calendar/Resources/DownArrowHighlight.tga" );
		self.hoursField:SetForeColor( Turbine.UI.Color( 1, 1, 0 ) );
	end
	self.hoursDown.MouseLeave = function( sender, args )
		sender:SetBackground( "FrostyPlugins/Calendar/Resources/DownArrow.tga" );
		self.hoursField:SetForeColor( Turbine.UI.Color( 1, 1, 1 ) );
	end

	self.hoursDown.Click = function( sender, args )
		self:AdjustHours( -1 );
	end
  
	self.hoursField = Turbine.UI.Label();
	self.hoursField:SetFont( Turbine.UI.Lotro.Font.Verdana16 );
	self.hoursField:SetParent( self );
	self.hoursField:SetPosition( 255, 40 );
	self.hoursField:SetMultiline( false );	
	self.hoursField:SetSelectable( false );	
	self.hoursField:SetSize( 20, 20 );
	self.hoursField:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleCenter );
	self.hoursField:SetVisible( true );

	self.colonField = Turbine.UI.Label();
	self.colonField:SetFont( Turbine.UI.Lotro.Font.Verdana16 );
	self.colonField:SetParent( self );
	self.colonField:SetPosition( 275, 40 );
	self.colonField:SetMultiline( false );	
	self.colonField:SetSelectable( false );	
	self.colonField:SetSize( 5, 20 );
	self.colonField:SetText( ":" );
	self.colonField:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleCenter );
	self.colonField:SetVisible( true );

	self.minutesField = Turbine.UI.Label();
	self.minutesField:SetFont( Turbine.UI.Lotro.Font.Verdana16 );
	self.minutesField:SetParent( self );
	self.minutesField:SetPosition( 280, 40 );
	self.minutesField:SetMultiline( false );	
	self.minutesField:SetSelectable( false );	
	self.minutesField:SetSize( 20, 20 );
	self.minutesField:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleCenter );
	self.minutesField:SetVisible( true );

	self.minutesUp = Turbine.UI.Button();
	self.minutesUp:SetBackground( "FrostyPlugins/Calendar/Resources/UpArrow.tga" );
	self.minutesUp:SetBlendMode( Turbine.UI.BlendMode.Overlay );
	self.minutesUp:SetParent( self );
	self.minutesUp:SetPosition( 300, 30 );
	self.minutesUp:SetSize( 20, 20 );
	self.minutesUp:SetVisible( true );

	self.minutesUp.MouseEnter = function( sender, args )
		sender:SetBackground( "FrostyPlugins/Calendar/Resources/UpArrowHighlight.tga" );
		self.minutesField:SetForeColor( Turbine.UI.Color( 1, 1, 0 ) );
	end
	self.minutesUp.MouseLeave = function( sender, args )
		sender:SetBackground( "FrostyPlugins/Calendar/Resources/UpArrow.tga" );
		self.minutesField:SetForeColor( Turbine.UI.Color( 1, 1, 1 ) );
	end

	self.minutesUp.Click = function( sender, args )
		self:AdjustMinutes( 1 );
	end
	
	self.minutesDown = Turbine.UI.Button();
	self.minutesDown:SetBackground( "FrostyPlugins/Calendar/Resources/DownArrow.tga" );
	self.minutesDown:SetBlendMode( Turbine.UI.BlendMode.Overlay );
	self.minutesDown:SetParent( self );
	self.minutesDown:SetPosition( 300, 50 );
	self.minutesDown:SetSize( 20, 20 );
	self.minutesDown:SetVisible( true );

  self.minutesDown.MouseEnter = function( sender, args )
	  sender:SetBackground( "FrostyPlugins/Calendar/Resources/DownArrowHighlight.tga" );
	  self.minutesField:SetForeColor( Turbine.UI.Color( 1, 1, 0 ) );
	end
  self.minutesDown.MouseLeave = function( sender, args )
	  sender:SetBackground( "FrostyPlugins/Calendar/Resources/DownArrow.tga" );
	  self.minutesField:SetForeColor( Turbine.UI.Color( 1, 1, 1 ) );
	end

  self.minutesDown.Click = function( sender, args )
		self:AdjustMinutes( -1 );
	end
  
  -- fixme: frosty
  --        alarm button?
  --        notify button?

  -- create the Description label and field
  self.descriptionLabel = Turbine.UI.Label();
  self.descriptionLabel:SetFont( Turbine.UI.Lotro.Font.Verdana16 );
  self.descriptionLabel:SetParent( self );
	self.descriptionLabel:SetPosition( 20, 60 );
	self.descriptionLabel:SetMultiline( false );	
	self.descriptionLabel:SetSelectable( false );	
	self.descriptionLabel:SetSize( 160, 20 );
	self.descriptionLabel:SetText( "Description" );
	self.descriptionLabel:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleLeft );
	self.descriptionLabel:SetVisible( true );

  self.descriptionField = Turbine.UI.Lotro.TextBox();
  self.descriptionField:SetFont( Turbine.UI.Lotro.Font.Verdana16 );
  self.descriptionField:SetParent( self );
	self.descriptionField:SetPosition( 20, 80 );
	self.descriptionField:SetMultiline( true );	
	self.descriptionField:SetSelectable( true );
	self.descriptionField:SetSize( 360, 120 );
	self.descriptionField:SetTextAlignment( Turbine.UI.ContentAlignment.TopLeft );
	self.descriptionField:SetVisible( true );

	-- create the save button
	self.buttonSave = Turbine.UI.Lotro.Button();
	self.buttonSave:SetParent( self );
	self.buttonSave:SetPosition( 40, 210 );
	self.buttonSave:SetSize( 80, 25 );
	self.buttonSave:SetText( "Save" );
	self.buttonSave:SetVisible( true );

  self.buttonSave.Click = function( sender, args )
 	  self:Save();
	end

	-- create the clear button
	self.buttonClear = Turbine.UI.Lotro.Button();
	self.buttonClear:SetParent( self );
	self.buttonClear:SetPosition( 195, 210 );
	self.buttonClear:SetSize( 80, 25 );
	self.buttonClear:SetText( "Clear" );
	self.buttonClear:SetVisible( true );

  self.buttonClear.Click = function( sender, args )
 	  self:Clear();
	end
  
	-- create the cancel button
	self.buttonCancel = Turbine.UI.Lotro.Button();
	self.buttonCancel:SetParent( self );
	self.buttonCancel:SetPosition( 280, 210 );
	self.buttonCancel:SetSize( 80, 25 );
	self.buttonCancel:SetText( "Cancel" );
	self.buttonCancel:SetVisible( true );

  self.buttonCancel.Click = function( sender, args )
 	  self:Cancel();
	end
  
	-- make sure we listen for key presses
	self:SetWantsKeyEvents( true );

  -- --------------------------------------------
  -- event handling
  -- --------------------------------------------
  
  --
  -- if the escape key is pressed, assume it is a "cancel"
  --
	self.KeyDown = function( sender, args )
		if ( args.Action == Turbine.UI.Lotro.Action.Escape ) then
			sender:Cancel();
		end
	end

  --
  -- if the position changes, save the new window location
  --
	self.PositionChanged = function( sender, args )
		local x, y = self:GetPosition();
		self.settings.positionX = x;
		self.settings.positionY = y;
		self:SaveSettings();
	end
end

-- --------------------------------------------------------------
-- AdjustHours
--
-- Description: the user has pressed one of the hours buttons
-- (up or down).  This function is used to bounds check the number
-- of hours and update the displayed hours field.
--
-- Params:
-- delta - integer - the amount of time to change the hours by.
--                   this will be 1 or -1 hours
-- --------------------------------------------------------------
function CalendarEntryEditor:AdjustHours( delta )
  self.hours = self.hours + delta;
  if( self.hours < 0 ) then
    self.hours = 23;
  end
  if( self.hours > 23 ) then
    self.hours = 0;
  end

  local newHoursLabel = tostring( self.hours );
  if( self.hours < 10 ) then
    newHoursLabel = "0" .. self.hours;
  end
  self.hoursField:SetText( newHoursLabel );
end

-- --------------------------------------------------------------
-- AdjustMinutes
--
-- Description: the user has pressed one of the minutes buttons
-- (up or down).  This function is used to bounds check the number
-- of minutes and update the displayed minutes field.
--
-- Params:
-- delta - integer - the amount of time to change the minues by.
--                   this will be 1 or -1 minutes
-- --------------------------------------------------------------
function CalendarEntryEditor:AdjustMinutes( delta )
  self.minutes = self.minutes + delta;
  if( self.minutes < 0 ) then
    self.minutes = 59;
  end
  if( self.minutes > 59 ) then
    self.minutes = 0;
  end

  local newMinutesLabel = tostring( self.minutes );
  if( self.minutes < 10 ) then
    newMinutesLabel = "0" .. self.minutes;
  end
  self.minutesField:SetText( newMinutesLabel );
end

-- --------------------------------------------------------------
-- Save
--
-- Description: the user has pressed the "Save" button.  Notify
-- the calendar that the enclosed CalendarEntry should be saved.
-- --------------------------------------------------------------
function CalendarEntryEditor:Save()
  -- save the settings - notify the creator that the data is ready  
  if( self.notifyCalendar ) then
    self.notifyCalendar:NotifySaveEntry( self );
  end
  
  self:SetVisible( false );
end

-- --------------------------------------------------------------
-- Cancel
--
-- Description: the user has pressed the "Cancel" button.  There
-- is nothing to do but hide the window.
-- --------------------------------------------------------------
function CalendarEntryEditor:Cancel()
  -- turn ourselves invisible, nothing else to do
  self:SetVisible( false );
end

-- --------------------------------------------------------------
-- Clear
--
-- Description: the user has pressed the "Clear" button.  Notify
-- the calendar that the enclosed CalendarEntry should be cleared.
-- --------------------------------------------------------------
function CalendarEntryEditor:Clear()
  -- clear the current entry - notify the creator that the data should be deleted
  if( self.notifyCalendar ) then
    self.notifyCalendar:NotifyClearEntry( self );
  end
  
  self:SetVisible( false );
end

-- --------------------------------------------------------------
-- InitializeEntry
--
-- Description: the entry is used to populate the fields on the editor.
-- If the user hits Save or Clear the input calendar is used to notify
-- of data changes.
-- 
-- Params:
-- entry - a CalendarEntry passed in from the calendar
-- calendar - the CalendarWindow that owns this object
-- --------------------------------------------------------------
function CalendarEntryEditor:InitializeEntry( entry, calendar )
  -- the user has clicked on a date entry - populate our fields with the data
  -- that they have specified and update the screen
  -- note: it is assumed that the entry.DateEntry is 5 characters, and in
  --       the format "12:59" or "03:45"
  if( string.len( entry.TimeEntry) ~= 5 ) then
    self.hours = "12";
    self.minutes = "59";
  else
    self.hours = string.sub( entry.TimeEntry, 1, 2 );
    self.minutes = string.sub( entry.TimeEntry, 4, 5 );
  end

  -- I cheat here by adjusting the hours and minutes by "0", which will cause
  -- them to redisplay using their new values
  self:AdjustHours( 0 );
  self:AdjustMinutes( 0 );

  self.dateLabel:SetText( "Date: " .. entry.DateEntry );  
  self.descriptionField:SetText( entry.Description );
  
  self.notifyCalendar = calendar;
  self.calendarEntry = entry;

  -- select all the text and make the field have focus
  -- fixme: frosty
  --        API does not yet support focus setting
  --        * without focus, placing the cursor in the field
  --          will remove the selection - boo!
  -- self.descriptionField:SelectAll();
  
  -- now show ourselves
  self:SetVisible( true );
end

-- --------------------------------------------------------------
-- LoadSettings
--
-- Description: loads our internal settings from disk.  For this class,
-- this includes the position of the editor window.  if the position does
-- not exist, we attempt to place it in the center of the window.
-- --------------------------------------------------------------
function CalendarEntryEditor:LoadSettings()
	-- load the settings.  If a value is not available, set a default value
	self.settings = Turbine.PluginData.Load( Turbine.DataScope.Character, "CalendarEntrySettings" );

	if ( type( self.settings ) ~= "table" ) then
		self.settings = { };
	end

	if ( not self.settings.positionX ) then
		self.settings.positionX = (Turbine.UI.Display.GetWidth() / 2) - (self:GetWidth() / 2 ); 
	end

	if ( not self.settings.positionY ) then
		self.settings.positionY = (Turbine.UI.Display.GetHeight() / 2) - (self:GetHeight() / 2 );
	end
end

-- --------------------------------------------------------------
-- SaveSettings
--
-- Description: save our internal settings to disk.  For this class,
-- this includes the position of the editor window
-- --------------------------------------------------------------
function CalendarEntryEditor:SaveSettings()
	-- save the settings
	Turbine.PluginData.Save( Turbine.DataScope.Character, "CalendarEntrySettings", self.settings );
end

